#include "test.h"

//#define OCSP_TEST 1
#define K_PRIKEY_LEN 100

static void CheckQuitKey(void * dummy);
static void EccSafeengineThread(void * arg);

static void TestSign();
static void TestSKFdeviceInit();
static void TestSKFdeviceGenKeyPair();
static void TestEncodeCert();
static void TestEnvelope();
static void TestVerifyCert();
static void TestBASE64EncDec();
static void TestVerifCertOnline();
static void TestDecodeCert();
static void TestEncDec();
static void TestGetCertInfoByOID();
static void TestEncDecPkcs12();
static void TestGetCertOnline();
static void TestChangePassword();
static void TestMultiThread();
static void TestDeviceInfo();
static void TestSupportedDeviceInfo();
static void TestWriteData2Device();
static void TestEncDecPkcs10();
static void TestShowVersion();
static void TestGetPublicKey();
static void TestUnLockDevice();
static void TestGetDataFromDevice();
static void TestVerifCertOnlineEx();
static void TestDecodeP10();
static void TestMutiEncDecP10();
static void TestLargeDataEncDec();

static void TestSignFile();
static void TestVerifySignFile();
static void TestEncEnvelopeFile();
static void TestDecEnvelopeFile();


int bRepeat;

typedef struct _ECC_SE_MT_STRUCT
{
	int num;
	unsigned long pritype;					/* sign */
	unsigned char priparm[256];
	unsigned char pripass[K_PRIKEY_LEN];
	unsigned long certtype;
	unsigned char certparm[256];
	unsigned long pritypeEnc;				/* enc */
	unsigned char priparmEnc[256];
	unsigned char pripassEnc[K_PRIKEY_LEN];
	unsigned long certtypeEnc;
	unsigned char certparmEnc[256];
	unsigned long certchaintype;
	unsigned char certchainparm[256];
	unsigned int mttestmode;
}ECC_SE_MT_STRUCT;

static void TestMutiEncDecP10()
{
	HSE hSE = NULL;	
	int testnum = 0;
	long retcode = -1;
	int priKeyType=0;
	unsigned char pubkey[1024]={0}, prikey[1024]={0};
	int pubkeylen=1024, prikeylen=1024;
	unsigned short usPubkeylen,usPrikeylen=1024;
	char priKeyParam[260]={0};
	char g_prikeydecpass[K_PRIKEY_LEN]={0};
	int dKeyPairType = 0; /* [0:sign];[1:enc] */
	/* sd */
	PKCS10_INFO s_CertInfoEx={0};
	PKCS10_INFO s_CertInfoExChk={0};
	int certinfodatalen = sizeof(g_certinfodata);
	unsigned short s_certinfodatalen = sizeof(g_certinfodata);
	unsigned long ulcertinfodatalen =  sizeof(g_certinfodata);
	char filepath[256]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key user password:");
	scanf("%s", g_prikeydecpass);

	for(testnum=0;testnum<1000;testnum++)
	{
		dKeyPairType = (priKeyType & 0x00FF0000) >> 16;
		retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
		if (retcode!=0) goto end;
		
		retcode = GetPublickey(priKeyType,priKeyParam,pubkey,&usPubkeylen);
		if (retcode!=0) goto end;	
		memcpy(s_CertInfoEx.subjectPublicKeyInfo,pubkey,usPubkeylen);
		s_CertInfoEx.subjectPublicKeyInfoSize = usPubkeylen;

		strncpy(s_CertInfoEx.subject_commonName,"SHECA_TEST",MAX_X500_NAME_LEN);
		strncpy(s_CertInfoEx.subject_organizationName,"SHECA",MAX_X500_NAME_LEN);
		strncpy(s_CertInfoEx.subject_stateOrProvinceName,"Ϻ",MAX_X500_NAME_LEN);
		strncpy(s_CertInfoEx.subject_countryName,"CN",MAX_X500_NAME_LEN);

		s_CertInfoEx.version = 3;

		retcode = ESE_EncodePkcs10(hSE,&s_CertInfoEx,ISSUER_NAME_PRINTABLE, SE_ALG_ecsm2WithSM3,g_certinfodata,&certinfodatalen);
		if (retcode!=0)
		{
			printf("[%d]ESE_EncodePkcs10 NG.\n",testnum);
			goto end;
		}		
		ulcertinfodatalen = certinfodatalen;
		//retcode = File_writeEx("e:\\muti_p10.dat",g_certinfodata,ulcertinfodatalen);

		retcode = ESE_DecodePkcs10(hSE,g_certinfodata,certinfodatalen,&s_CertInfoExChk);
		if (retcode!=0)
		{
			printf("[%d]ESE_DecodePkcs10 NG.\n",testnum);
			goto end;
		}
		
		(retcode!=0) ? printf("[%d]TestEncDecPkcs10 NG.[0x%x]\n",testnum,retcode) : printf("[%d]TestEncDecPkcs10 OK.\n",testnum);
		ESE_ClearSession(hSE);
		hSE = NULL;
		memset(g_certinfodata,0,sizeof(g_certinfodata));
		certinfodatalen = sizeof(g_certinfodata);
	}

end:
	(retcode!=0) ? printf("[%d]TestEncDecPkcs10 NG.[0x%x]\n",testnum,retcode) : printf("[%d]TestEncDecPkcs10 OK.\n",testnum);
	ESE_ClearSession(hSE);
	return;

}

static void TestGetDataFromDevice()
{
	long retcode = -1;
	int devType = 0;
	char devParam[260] = {0};
	char strSavepath[256] = {0};
	unsigned char temp[4096]={0};
	unsigned short usTempsize = 4096;

	printf("\ninput device type(HEX):");
	scanf("%x", &devType);
	printf("\ninput device parameter:");
	scanf("%s", devParam);

	retcode = GetCertificateChain(devType,devParam,temp,&usTempsize);
	if (retcode != 0) goto end;

	memset( strSavepath , 0 , 256 );
	printf("\ninput save path:");
	scanf("%s", strSavepath);
		
	retcode = File_writeEx( strSavepath, temp, (unsigned long)usTempsize );
end:
	(retcode!=0) ? printf("TestGetDataFromDevice NG.[0x%x]\n",retcode) : printf("TestGetDataFromDevice OK.\n");
	return;
}

static void TestUnLockDevice()
{
	long retcode = -1;

	int priKeyType=0;
	char priKeyParam[260]={0};
	char adminpass[20]={0};
	char newuserpass[20]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key admin password:");
	scanf("%s", adminpass);
	printf("\ninput private key new user password:");
	scanf("%s", newuserpass);

	retcode = UnlockDevice(priKeyType,priKeyParam,adminpass,newuserpass);
	(retcode!=0) ? printf("TestUnLockDevice NG.[0x%x]\n",retcode) : printf("TestUnLockDevice OK.\n");
	return;
}

static void TestShowVersion()
{
	long retcode = -1;
	long lm=0,lv=0;
	retcode = ESE_ShowVersion(NULL,&lm,&lv);
	printf("The current version is 0x%x,0x%x\n\n",lm,lv);
}

static void TestGetPublicKey()
{
	HSE hSE = NULL;	
	long retcode = -1;

	int priKeyType=0;
	
	unsigned char pubkey[1024]={0};
	int pubkeylen=1024;
	unsigned char pubkeychk[1024]={0};
	int pubkeychklen=1024;

	unsigned short usPubkeylen;
	char priKeyParam[260]={0};
	char g_prikeydecpass[K_PRIKEY_LEN]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key user password:");
	scanf("%s", g_prikeydecpass);

	retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
	if (retcode!=0) goto end;

	retcode = GetPublickey(priKeyType,priKeyParam,pubkey,&usPubkeylen);
	if (retcode!=0) goto end;

	retcode = ESE_GetPublicKey(hSE,"",pubkeychk,&pubkeychklen);
	if (retcode!=0) goto end;

	retcode = (pubkeychklen == usPubkeylen && memcmp(pubkey,pubkeychk,pubkeychklen) == 0) ? 0 : -1;
	if (retcode != 0) goto end;

end:
	(retcode!=0) ? printf("TestGetPublicKey NG.[0x%x]\n",retcode) : printf("TestGetPublicKey OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestDecodeP10()
{
	HSE hSE = NULL;	
	long retcode = -1;
	int priKeyType=0;
	unsigned char pubkey[1024]={0}, prikey[1024]={0};
	int pubkeylen=1024, prikeylen=1024;
	unsigned short usPubkeylen,usPrikeylen=1024;
	char priKeyParam[260]={0};
	char g_prikeydecpass[K_PRIKEY_LEN]={0};
	int dKeyPairType = 0; /* [0:sign];[1:enc] */
	/* sd */
	PKCS10_INFO s_CertInfoEx={0};
	PKCS10_INFO s_CertInfoExChk={0};
	int certinfodatalen = sizeof(g_certinfodata);
	unsigned short s_certinfodatalen = sizeof(g_certinfodata);
	unsigned long ulcertinfodatalen =  sizeof(g_certinfodata);
	char filepath[256]={0};

	unsigned char pemout[4096]={0};
	unsigned long pemoutlen = 4096;

	int ispem = 0;

//	printf("\ninput private key type(HEX):");
//	scanf("%x", &priKeyType);
//	printf("\ninput private key parameter:");
//	scanf("%s", priKeyParam);
//	printf("\ninput private key user password:");
//	scanf("%s", g_prikeydecpass);
	printf("\ninput p10 file path:");
	scanf("%s", filepath);

	printf("\nis PEM encode:");
	scanf("%d", &ispem);

	//retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
	retcode = ESE_InitialSession(&hSE,2,"","",0,2,"","");
	if (retcode!=0) goto end;

	retcode = ReadDiskFile(filepath,g_certinfodata,&ulcertinfodatalen);
	if (retcode!=0) goto end;

	if (ispem != 0)
	{
		retcode = ESE_PEMDecode(hSE,g_certinfodata,ulcertinfodatalen,pemout,&pemoutlen);
		if (retcode!=0) goto end;
	}
	else
	{
		pemoutlen = ulcertinfodatalen;
		memcpy(pemout,g_certinfodata,pemoutlen);
	}	
	certinfodatalen = pemoutlen;
	retcode = ESE_DecodePkcs10(hSE,pemout,certinfodatalen,&s_CertInfoExChk);
	if (retcode!=0) goto end;

end:
	(retcode!=0) ? printf("TestDecodeP10 NG.[0x%x]\n",retcode) : printf("TestDecodeP10 OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestEncDecPkcs10()
{
	HSE hSE = NULL;	
	long retcode = -1;
	int priKeyType=0;
	unsigned char pubkey[1024]={0}, prikey[1024]={0};
	int pubkeylen=1024, prikeylen=1024;
	unsigned short usPubkeylen,usPrikeylen=1024;
	char priKeyParam[260]={0};
	char g_prikeydecpass[K_PRIKEY_LEN]={0};
	int dKeyPairType = 0; /* [0:sign];[1:enc] */
	/* sd */
	PKCS10_INFO s_CertInfoEx={0};
	PKCS10_INFO s_CertInfoExChk={0};
	int certinfodatalen = sizeof(g_certinfodata);
	unsigned short s_certinfodatalen = sizeof(g_certinfodata);
	unsigned long ulcertinfodatalen =  sizeof(g_certinfodata);
	char filepath[256]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key user password:");
	scanf("%s", g_prikeydecpass);

	dKeyPairType = (priKeyType & 0x00FF0000) >> 16;
	retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
	if (retcode!=0) goto end;
	
	retcode = GetPublickey(priKeyType,priKeyParam,pubkey,&usPubkeylen);
	if (retcode!=0) goto end;	
	memcpy(s_CertInfoEx.subjectPublicKeyInfo,pubkey,usPubkeylen);
	s_CertInfoEx.subjectPublicKeyInfoSize = usPubkeylen;

#if 0
	printf("\ninput common name:");
	scanf("%s", s_CertInfoEx.subject_commonName);
	/*strncpy(s_CertInfoEx.subject_commonName,"veniwang",8);*/
	printf("\ninput country name:");
	scanf("%s", s_CertInfoEx.subject_countryName);
	/*strncpy(s_CertInfoEx.subject_countryName,"й",4);*/
	printf("\ninput organization name:");
	scanf("%s", s_CertInfoEx.subject_organizationName);
	/*strncpy(s_CertInfoEx.subject_organizationName,"sheca",5);*/
	printf("\ninput organizational Unitname:");
	scanf("%s", s_CertInfoEx.subject_organizationalUnitname1);
	/*strncpy(s_CertInfoEx.subject_organizationalUnitname1,"ϺCA",6);*/
	printf("\ninput stateOrProvince name:");
	scanf("%s", s_CertInfoEx.subject_stateOrProvinceName);
	/*strncpy(s_CertInfoEx.subject_stateOrProvinceName,"Ϻ",4);*/
	printf("\ninput locality name:");
	scanf("%s", s_CertInfoEx.subject_localityName);
	/*strncpy(s_CertInfoEx.subject_localityName,"shanghai",8);*/
	printf("\ninput title name:");
	scanf("%s", s_CertInfoEx.subject_title);
	/*strncpy(s_CertInfoEx.subject_title,"ABC",3);*/
	printf("\ninput surname:");
	scanf("%s", s_CertInfoEx.subject_surname);
	/*strncpy(s_CertInfoEx.subject_surname,"wang",4);*/
	printf("\ninput give name:");
	scanf("%s", s_CertInfoEx.subject_givenName);
	/*strncpy(s_CertInfoEx.subject_givenName,"yulin",5);*/
	printf("\ninput initials name:");
	scanf("%s", s_CertInfoEx.subject_initials);
	/*strncpy(s_CertInfoEx.subject_initials,"XYZ",3);*/
	printf("\ninput email:");
	scanf("%s", s_CertInfoEx.subject_email);
	/*strncpy(s_CertInfoEx.subject_email,"abc@126.com",11);*/
	printf("\ninput postal address:");
	scanf("%s", s_CertInfoEx.subject_postalAddress);
	/*strncpy(s_CertInfoEx.subject_postalAddress,"hongkou",7);*/
	printf("\ninput postal code:");
	scanf("%s", s_CertInfoEx.subject_postalCode);
	/*strncpy(s_CertInfoEx.subject_postalCode,"200080",6);*/
	printf("\ninput postal OfficeBox:");
	scanf("%s", s_CertInfoEx.subject_postalOfficeBox);
	/*strncpy(s_CertInfoEx.subject_postalOfficeBox,"Ĵ·",8);*/
	printf("\ninput telephone number:");
	scanf("%s", s_CertInfoEx.subject_telephoneNumber);
	/*strncpy(s_CertInfoEx.subject_telephoneNumber,"36390000",8);*/
	printf("\ninput telex number:");
	scanf("%s", s_CertInfoEx.subject_telexNumber);
	/*strncpy(s_CertInfoEx.subject_telexNumber,"36390001",8);*/
#endif

	strncpy(s_CertInfoEx.subject_commonName,"Ϻбɽҵ                             (Ϻбɽѯ)",MAX_X500_NAME_LEN);
	strncpy(s_CertInfoEx.subject_organizationName,"Ϻбɽҵ                             (Ϻбɽѯ)",MAX_X500_NAME_LEN);
	strncpy(s_CertInfoEx.subject_stateOrProvinceName,"Ϻ",MAX_X500_NAME_LEN);
	strncpy(s_CertInfoEx.subject_countryName,"CN",MAX_X500_NAME_LEN);

	s_CertInfoEx.version = 3;

/*
	CHKINPUT(s_CertInfoEx.subject_countryName);
	CHKINPUT(s_CertInfoEx.subject_organizationName);
	CHKINPUT(s_CertInfoEx.subject_organizationalUnitname1);
	CHKINPUT(s_CertInfoEx.subject_stateOrProvinceName);
	CHKINPUT(s_CertInfoEx.subject_localityName);
	CHKINPUT(s_CertInfoEx.subject_title);
	CHKINPUT(s_CertInfoEx.subject_surname);
	CHKINPUT(s_CertInfoEx.subject_givenName);
	CHKINPUT(s_CertInfoEx.subject_initials);
	CHKINPUT(s_CertInfoEx.subject_email);
	CHKINPUT(s_CertInfoEx.subject_postalAddress);
	CHKINPUT(s_CertInfoEx.subject_postalOfficeBox);
	CHKINPUT(s_CertInfoEx.subject_telephoneNumber);
	CHKINPUT(s_CertInfoEx.subject_telexNumber);
*/
	retcode = ESE_EncodePkcs10(hSE,&s_CertInfoEx,ISSUER_NAME_PRINTABLE, SE_ALG_ecsm2WithSM3,g_certinfodata,&certinfodatalen);
	if (retcode!=0) goto end;
	
	retcode = ESE_DecodePkcs10(hSE,g_certinfodata,certinfodatalen,&s_CertInfoExChk);
	if (retcode!=0) goto end;
	
	printf("\ninput save pkcs10 file path:");
	scanf("%s", filepath);
	ulcertinfodatalen = certinfodatalen;
	retcode = File_writeEx(filepath,g_certinfodata,ulcertinfodatalen);

end:
	(retcode!=0) ? printf("TestEncDecPkcs10 NG.[0x%x]\n",retcode) : printf("TestEncDecPkcs10 OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestWriteData2Device()
{
	HSE hSE = NULL;
	long retcode = -1;
	long priKeyType = 0;
	char priKeyParam[200] = {0};
	char priPassword[20] = {0};
	char certpath[256];
	char certchainpath[256];
	char cardinfopath[256];

	unsigned char certdata[2048]={0};
	unsigned short uscertlen = sizeof(certdata);
	unsigned long ulcertlen = sizeof(certdata);
	unsigned char certdatachk[2048]={0};
	unsigned short uscertchklen = sizeof(certdatachk);
	unsigned long ulcertchklen = sizeof(certdatachk);

	unsigned char certchaindata[2048]={0};
	unsigned short uscertchainlen = sizeof(certchaindata);
	unsigned long ulcertchainlen = sizeof(certchaindata);
	unsigned char certchaindatachk[2048]={0};
	unsigned short uscertchainchklen = sizeof(certchaindatachk);
	unsigned long ulcertchainchklen = sizeof(certchaindatachk);

	unsigned char cardinfodata[1024]={0};
	unsigned short uscardinfodatalen = sizeof(cardinfodata);
	unsigned long ulcardinfodatalen = sizeof(cardinfodata);

	unsigned char cardinfodatachk[1024]={0};
	unsigned short uscardinfodatachklen = sizeof(cardinfodatachk);
	unsigned long ulcardinfodatachklen = sizeof(cardinfodatachk);

	printf("\ninput device type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput device parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput password:");
	scanf("%s", priPassword);
	printf("\ninput cert path:");
	scanf("%s", certpath);
	printf("\ninput certchain path:");
	scanf("%s", certchainpath);
	printf("\ninput cardinfo path:");
	scanf("%s", cardinfopath);

	CHKINPUT(certpath);
	CHKINPUT(certchainpath);
	CHKINPUT(cardinfopath);

	/* write certificate */
	if (strlen(certpath)>0)
	{
		retcode = ReadDiskFile(certpath,certdata,&ulcertlen);
		if (retcode != 0) goto end;
		uscertlen = ulcertlen;
		retcode = WriteCertificate(priKeyType,priKeyParam,priPassword,certdata,uscertlen);
		if (retcode != 0) goto end;
		retcode = GetCertificate(priKeyType,priKeyParam,certdatachk,&uscertchklen);
		if (retcode != 0) goto end;
		retcode = (uscertlen == uscertchklen && memcmp(certdata,certdatachk,uscertchklen) == 0) ? 0 : -1;
		if (retcode != 0) goto end;
	}		
	/* write certificate chain */
	if (strlen(certchainpath)>0)
	{
		retcode = ReadDiskFile(certchainpath,certchaindata,&ulcertchainlen);
		if (retcode != 0) goto end;
		uscertchainlen = ulcertchainlen;
		retcode = WriteCertificateChain(priKeyType,priKeyParam,priPassword,certchaindata,uscertchainlen);
		if (retcode != 0) goto end;
		retcode = GetCertificateChain(priKeyType,priKeyParam,certchaindatachk,&uscertchainchklen);
		if (retcode != 0) goto end;
		retcode = (uscertchainlen == uscertchainchklen && memcmp(certchaindata,certchaindatachk,uscertchainchklen) == 0) ? 0 : -1;
		if (retcode != 0) goto end;
	}
	/* write card info */
	if (strlen(cardinfopath)>0)
	{
		retcode = ReadDiskFile(cardinfopath,cardinfodata,&ulcardinfodatalen);
		if (retcode != 0) goto end;
		uscardinfodatalen = ulcardinfodatalen;
		retcode = WriteCardInfo(priKeyType,priKeyParam,priPassword,cardinfodata,uscardinfodatalen);
		if (retcode != 0) goto end;
		retcode = GetCardInfo(priKeyType,priKeyParam,cardinfodatachk,&uscardinfodatachklen);
		if (retcode != 0) goto end;
		retcode = (uscardinfodatalen == uscardinfodatachklen && memcmp(cardinfodata,cardinfodatachk,uscardinfodatachklen) == 0) ? 0 : -1;
		if (retcode != 0) goto end;
	}

end:
	(retcode!=0) ? printf("TestWriteData2Device NG.[0x%x]\n",retcode) : printf("TestWriteData2Device OK.\n");
	return;
}

static void TestSupportedDeviceInfo()
{
	long retcode = 0;
	int dDevNums = 0;
	DEVICE_LIST* tdevList = NULL;
	int i=0,j=0;
	int portnum = 0;
	char portlist[8] = {0};
	int portlist_size = 8;

	retcode = ESE_ListPlugDevice(NULL,&dDevNums);
	if (dDevNums > 0)
	{
		tdevList = (DEVICE_LIST*)malloc(dDevNums*sizeof(DEVICE_LIST));
		retcode = ESE_ListPlugDevice(tdevList,&dDevNums);
		printf("device num = %d\n",dDevNums);
		for (i=0;i<dDevNums;i++)
		{
			printf("devType=0x%x,alg=%d,devname=%s,devparam=%s\n",tdevList->devices[i].devType,tdevList->devices[i].alg,tdevList->devices[i].devName,tdevList->devices[i].devParam);
		}
	}
	else
	{
		printf("has no device.\n");
	}

	if (tdevList) free(tdevList);
}

static void TestDeviceInfo()
{
	long retcode = 0;
	int dDevNums = 0;
	DEVICE_LIST* tdevList = NULL;
	int i=0,j=0;
	int portnum = 0;
	char portlist[8] = {0};
	int portlist_size = 8;;

	retcode = ESE_ListAllDevice(NULL,&dDevNums);
	if (dDevNums > 0)
	{
		tdevList = (DEVICE_LIST*)malloc(dDevNums*sizeof(DEVICE_LIST));
		retcode = ESE_ListAllDevice(tdevList,&dDevNums);
		printf("device num = %d\n",dDevNums);
		for (i=0;i<dDevNums;i++)
		{
			printf("devType=0x%x,alg=%d,devname=%s\n",tdevList->devices[i].devType,tdevList->devices[i].alg,tdevList->devices[i].devName);
			if (tdevList->devices[i].alg == 3 || tdevList->devices[i].alg == 2)
			{
				retcode = ESE_GetDevicePortList(tdevList->devices[i].devType,&portnum,portlist,portlist_size);
				if (retcode == 0)
				{
					for (j=0;j<portnum;j++)
						printf("port[%d] is deteced.\n",j);
				}							
			}

		}
	}
	else
	{
		printf("has no device.\n");
	}

	if (tdevList) free(tdevList);
}

static void CheckQuitKey(void * dummy)
{
	char c;

    while(bRepeat)
	{
#ifdef _WINDOWS
		c = tolower(_getch());
#else
		c = tolower(getchar());
#endif /* _WINDOWS */
		if(c=='q')
		{
			bRepeat = 0;
			break;
		}
	}
}

static void EccSafeengineThread(void * arg)
{
	HSE hSE_sign =  NULL;
	HSE hSE_enc =  NULL;

	long retcode = 0;

	int inum = 0;
	unsigned char out[4096];
	unsigned long outlength = sizeof(out);

	unsigned char cert[2048];
	unsigned short certlength = sizeof(cert);
	unsigned char certEnc[2048];
	unsigned short certEnclength = sizeof(certEnc);	

	unsigned char indata[] = "abc";
	unsigned long inlength = 3;
	unsigned char outdata[100];
	unsigned long outdatalen = sizeof(outdata);

	ECC_SE_MT_STRUCT* tparm = ((ECC_SE_MT_STRUCT*)arg);

	while (bRepeat)
	{
#ifdef _WINDOWS
		Sleep( 10 );
#else
		sleep( 1 );
#endif /* _WINDOWS */
		/* get sign cert handle */
		retcode = ESE_InitialSession(&hSE_sign,tparm->pritype,tparm->priparm,tparm->pripass,0,2,"","");
		if (retcode != 0){
			printf("thread[%d] ESE_InitialSession for sign NG.[0x%x]\n",tparm->num,retcode);
			ESE_ClearSession(hSE_sign);
			hSE_sign = NULL;
			ESE_ClearSession(hSE_enc);
			hSE_enc = NULL;
			continue;
		}else{
			printf("thread[%d] ESE_InitialSession for sign OK.\n",tparm->num);
		}
		/* get enc cert handle */
		retcode = ESE_InitialSession(&hSE_enc,tparm->pritypeEnc,tparm->priparmEnc,tparm->pripassEnc,0,2,"","");
		if (retcode != 0){
			printf("thread[%d] ESE_InitialSession for enc NG.[0x%x]\n",tparm->num,retcode);
			ESE_ClearSession(hSE_sign);
			hSE_sign = NULL;
			ESE_ClearSession(hSE_enc);
			hSE_enc = NULL;
			continue;
		}else{
			printf("thread[%d] ESE_InitialSession for enc OK.\n",tparm->num);
		}			
		
		/* get sign cert data */
		certlength = 2048;
		retcode = ESE_GetSelfCertificate(hSE_sign, tparm->certtype, tparm->certparm, cert, &certlength);
		if (retcode != 0){
			printf("thread[%d] ESE_GetSelfCertificate for sign NG.[0x%x]\n",tparm->num,retcode);
			ESE_ClearSession(hSE_sign);
			hSE_sign = NULL;
			ESE_ClearSession(hSE_enc);
			hSE_enc = NULL;
			continue;
		}else{
			printf("thread[%d] ESE_GetSelfCertificate for sign OK.\n",tparm->num);
		}
		/* get enc cert data */
		certEnclength = 2048;
		retcode = ESE_GetSelfCertificate(hSE_enc, tparm->certtypeEnc, tparm->certparmEnc, certEnc, &certEnclength);
		if (retcode != 0){
			printf("thread[%d] ESE_GetSelfCertificate for enc NG.[0x%x]\n",tparm->num,retcode);
			ESE_ClearSession(hSE_sign);
			hSE_sign = NULL;
			ESE_ClearSession(hSE_enc);
			hSE_enc = NULL;
			continue;
		}else{
			printf("thread[%d] ESE_GetSelfCertificate for enc OK.\n",tparm->num);
		}
		/* need to testing sign-verify? */
		if ((tparm->mttestmode & 0x00000001) == 1)
		{
			/* make signdata */
			outlength = 4096;
			retcode = ESE_SignData(hSE_sign, indata, inlength, SE_ALG_ecsm2WithSM3, out, &outlength);
			if (retcode != 0){
				printf("thread[%d] ESE_SignData NG.[0x%x]\n",tparm->num,retcode);
				ESE_ClearSession(hSE_sign);
				hSE_sign = NULL;
				ESE_ClearSession(hSE_enc);
				hSE_enc = NULL;
				continue;
			}else{
				printf("thread[%d] ESE_SignData OK.\n",tparm->num);
			}
			/* verify signdata */
			retcode = ESE_VerifySignData( hSE_sign, indata, inlength, SE_ALG_ecsm2WithSM3, out, outlength, cert, certlength );
			if (retcode != 0){
				printf("thread[%d] ESE_VerifySignData NG.[0x%x]\n",tparm->num,retcode);
				ESE_ClearSession(hSE_sign);
				hSE_sign = NULL;
				ESE_ClearSession(hSE_enc);
				hSE_enc = NULL;
				continue;
			}else{
				printf("thread[%d] ESE_VerifySignData OK.\n",tparm->num);
			}
		}
		/* need to testing enc-dec envelope? */
		if ((tparm->mttestmode & 0x00000002) == 2)
		{
			/* encode envelope */
			outlength = 4096;
			retcode = ESE_Envelope_P7(hSE_enc, indata, inlength, SE_ALG_sm1, SE_ALG_ecSM2_3Encryption, out, &outlength, certEnc, certEnclength);
			if (retcode != 0){
				printf("thread[%d] ESE_Envelope_P7 NG.[0x%x]\n",tparm->num,retcode);
				ESE_ClearSession(hSE_sign);
				hSE_sign = NULL;
				ESE_ClearSession(hSE_enc);
				hSE_enc = NULL;
				continue;
			}else{
				printf("thread[%d] ESE_Envelope_P7 OK.\n",tparm->num);
			}
			/* decode envelope */
			outdatalen = 100;
			retcode = ESE_DecodeEnvelope_P7(hSE_enc, out, outlength, outdata, &outdatalen);
			if (retcode != 0){
				printf("thread[%d] ESE_DecodeEnvelope_P7 NG.[0x%x]\n",tparm->num,retcode);
				ESE_ClearSession(hSE_sign);
				hSE_sign = NULL;
				ESE_ClearSession(hSE_enc);
				hSE_enc = NULL;
				continue;
			}else{
				printf("thread[%d] ESE_DecodeEnvelope_P7 OK.\n",tparm->num);
			}
			/* check enc dec envelope data */
			if (outdatalen != inlength || memcmp(outdata,indata,inlength) != 0) retcode = -1;
			if (retcode != 0)
			{
				printf("enc dec data not match.\n");
				ESE_ClearSession(hSE_sign);
				hSE_sign = NULL;
				ESE_ClearSession(hSE_enc);
				hSE_enc = NULL;
				continue;
			}		
		}
		/* test end. */	
		printf("thread[%d] testing...\n",tparm->num);
		ESE_ClearSession(hSE_sign);
		hSE_sign = NULL;
		ESE_ClearSession(hSE_enc);
		hSE_enc = NULL;
	}
	printf("thread[%d] exited.\n",tparm->num);
}

static void TestMultiThread()
{
	int i = 0;
	int threadnum = 0;
	/* sign */
	unsigned long tpritype = 0;
	unsigned char tpriparm[256]={0};
	unsigned char tpripass[K_PRIKEY_LEN]={0};
	unsigned long tcerttype=0;
	unsigned char tcertparm[256]={0};
	/* enc */
	unsigned long tpritypeEnc = 0;
	unsigned char tpriparmEnc[256]={0};
	unsigned char tpripassEnc[K_PRIKEY_LEN]={0};
	unsigned long tcerttypeEnc=0;
	unsigned char tcertparmEnc[256]={0};
	/* certchain */
	unsigned long tcertchaintype=0;
	unsigned char tcertchainparm[256]={0};
	/* multi-thread test mode */
	unsigned int tmmode = 0;
	/* sign cert */
	printf("\ninput device type of signcert[HEX]:");
	scanf("%x", &tpritype);
	printf("\ninput device parameter of signcert:");
	scanf("%s", tpriparm);
	printf("\ninput password of signcert:");
	scanf("%s", tpripass);
	printf("\ninput cert device type of signcert[HEX]:");
	scanf("%x", &tcerttype);
	printf("\ninput cert device parameter of signcert:");
	scanf("%s", tcertparm);
	/* enc cert */
	printf("\ninput device type of enccert[HEX]:");
	scanf("%x", &tpritypeEnc);
	printf("\ninput device parameter of enccert:");
	scanf("%s", tpriparmEnc);
	printf("\ninput password of enccert:");
	scanf("%s", tpripassEnc);
	printf("\ninput cert device type of enccert[HEX]:");
	scanf("%x", &tcerttypeEnc);
	printf("\ninput cert device parameter of enccert:");
	scanf("%s", tcertparmEnc);
	/* certchain sign */
	printf("\ninput certchain device type[HEX]:");
	scanf("%x", &tcertchaintype);
	printf("\ninput certchain device parameter:");
	scanf("%s", tcertchainparm);
	/* multi-thread mode */
	printf("\ninput multi-thread mode:");
	scanf("%x", &tmmode);
	/* thread numbers */
	printf("\ninput thread numbers:");
	scanf("%d", &threadnum);

	bRepeat = 1;
	_beginthread((void *)CheckQuitKey, 0, NULL);
	
	for (i=0;i<threadnum;i++)
	{
		ECC_SE_MT_STRUCT* tparma = (ECC_SE_MT_STRUCT*)malloc(sizeof(ECC_SE_MT_STRUCT)*threadnum);
		tparma[i].num = i+1;
		/* sign */
		tparma[i].pritype = tpritype;
		memcpy(tparma[i].priparm,tpriparm,256);
		memcpy(tparma[i].pripass,tpripass,K_PRIKEY_LEN);		
		tparma[i].certtype=tcerttype;
		memcpy(tparma[i].certparm,tcertparm,256);
		/* enc */
		tparma[i].pritypeEnc = tpritypeEnc;
		memcpy(tparma[i].priparmEnc,tpriparmEnc,256);
		memcpy(tparma[i].pripassEnc,tpripassEnc,K_PRIKEY_LEN);		
		tparma[i].certtypeEnc=tcerttypeEnc;
		memcpy(tparma[i].certparmEnc,tcertparmEnc,256);
		/* certchain */
		tparma[i].certchaintype=tcertchaintype;
		memcpy(tparma[i].certchainparm,tcertchainparm,256);
		/* multi-thread mode */
		tparma[i].mttestmode=tmmode;
#ifdef _WINDOWS
		Sleep(1000);
#else
		sleep(1);
#endif /* _WINDOWS */
		_beginthread((void *)EccSafeengineThread, 0, (void*)&tparma[i]);
	}

	while(bRepeat)
	{
#ifdef _WINDOWS
		Sleep(1000);
#else
		sleep(1);
#endif /* _WINDOWS */
	}
	printf("Quit.\n\n");
	return;
}

static void TestChangePassword()
{
	HSE hSE = NULL;
	long retcode = -1;
	char priKeyParam[260];
	int priKeyType;
	char oldpassword[20];
	char newpassword[20];
	int dType = 0;

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput old password:");
	scanf("%s", oldpassword);
	printf("\ninput new password:");
	scanf("%s", newpassword);
	printf("\ninput change type[0:user,1:admin,2:device]:");
	scanf("%d", &dType);

	retcode = ChangePassword(priKeyType,priKeyParam,oldpassword,newpassword,dType);
	if (retcode !=0) goto end;

end:
	(retcode!=0) ? printf("TestChangePassword NG.[0x%x]\n",retcode) : printf("TestChangePassword OK.\n");
	return;
}

static void TestGetCertOnline()
{
	printf("not support.\n");
}

static void TestEncDecPkcs12()
{
	printf("not support.\n");
}

void TestGetCertInfoByOID()
{
	HSE hSE = NULL;
	long retcode = -1;	
	int certdevtype;
	char certdevparam[200];
	unsigned char cert[2048]={0};
	unsigned short certlen = sizeof(cert);
	
	/* sp */
	unsigned char oiddata[128] = {0};
	unsigned char itemdata[1024]={0};
	unsigned short itemdatalen = sizeof(itemdata);

	printf("\nPlease input certificate device type[HEX]:");
	scanf("%x", &certdevtype);
	printf("\nPlease input certificate device parameter:");
	scanf("%s", certdevparam);
	printf("\nPlease input oid data:");
	scanf("%s", oiddata);

	retcode=ESE_InitialSession(&hSE, 2, "", "", 0, 2, "", "");
	if (retcode !=0) goto end;

	retcode = ESE_GetSelfCertificate(hSE, certdevtype, certdevparam, cert, &certlen);
	if (retcode !=0) goto end;

	retcode = ESE_GetCertInfoByOID(hSE,cert,certlen,oiddata,itemdata,&itemdatalen);
	if (retcode !=0) goto end;

	(itemdatalen > 0) ? printf("[oid:%s]=%s,len=%d\n",oiddata,itemdata,itemdatalen) : printf("[oid:%s]=NULL\n",oiddata);

end:
	(retcode!=0) ? printf("TestGetCertInfoByOID NG.[0x%x]\n",retcode) : printf("TestGetCertInfoByOID OK.\n");
	return;
}

static void TestEncDec()
{
	long retcode = 0;
	HSE hSE = NULL;
	HSE hSE_Enc = NULL;
	unsigned long devType = 0;
	char devParam[260]={0};
	char password[200]={0};
	unsigned char cert[2048]={0};
	unsigned short certlen = 2048;

	unsigned char randomdata[2048]={0};
	int randomdatalen=sizeof(randomdata);
	unsigned long ulrandomdatalen = sizeof(randomdata);

	unsigned char indata[4096]={0};
	unsigned long ulindatalen = sizeof(indata);
	unsigned char encdata[4096]={0};
	unsigned long ulencdatalen = sizeof(encdata);
	unsigned char outdata[4096]={0};
	unsigned long uloutdatalen = sizeof(outdata);
	unsigned long ulalgid = 0;
	char skeypath[MAX_PATH]={0};
	char encdatapath[MAX_PATH]={0};
	char externalEncCertPath[MAX_PATH]={0};

	printf("\ninput device type[HEX]:");
	scanf("%x", &devType);
	printf("\ninput device parameter:");
	scanf("%s", devParam);
	printf("\ninput password:");
	scanf("%s", password);
	printf("\ninput alg id([HEX],(eg:0x00000101[SM1],0x00000401[SM4],0x00000501[SDBI]):");
	scanf("%x", &ulalgid);
	printf("\ninput test data:");
	scanf("%s", indata);
	printf("\ninput external enc cert path:");
	scanf("%s", externalEncCertPath);

	if (ulalgid == 0x501)
	{
		printf("\ninput skey path:");
		scanf("%s", skeypath);		
		printf("\ninput enc data path:");
		scanf("%s", encdatapath);

		memset(randomdata,0,ulrandomdatalen);
		retcode = ReadDiskFile(skeypath,randomdata,&ulrandomdatalen);
		if (retcode!=0) goto end;
		memset(encdata,0,ulencdatalen);
		retcode = ReadDiskFile(encdatapath,encdata,&ulencdatalen);
		if (retcode!=0) goto end;
		retcode = ESE_SymDecrypt(NULL,randomdata,ulrandomdatalen,ulalgid,encdata,(int)ulencdatalen,outdata,(int*)&uloutdatalen);
		if (retcode!=0) goto end;
		retcode = File_writeEx("dec.dat",outdata,uloutdatalen);
		printf("TestEncDec OK.\n");
		return;
	}

	retcode = ESE_InitialSession(&hSE, devType, devParam, password, 0, 2, "", "");
	if (retcode!=0) goto end;

	/* test1 */
	randomdatalen = 16;
	retcode = ESE_GenRandomBytes(hSE,randomdata,randomdatalen);
	if (retcode!=0) goto end;	
	ulrandomdatalen = randomdatalen;

	ulindatalen = strlen(indata);
	/*retcode = ESE_SymEncrypt(hSE,randomdata,ulrandomdatalen,ulalgid,indata,ulindatalen,encdata,&ulencdatalen);*/
	retcode = ESE_InitialSession(&hSE_Enc, 2, "", "", 0, 2, "", "");
	retcode = ESE_SymEncrypt(hSE_Enc,randomdata,ulrandomdatalen,ulalgid,indata,ulindatalen,encdata,&ulencdatalen);
	ESE_ClearSession(hSE_Enc);

	if (retcode!=0) goto end;

	retcode = ESE_SymDecrypt(hSE,randomdata,ulrandomdatalen,ulalgid,encdata,ulencdatalen,outdata,&uloutdatalen);
	if (retcode!=0) goto end;

	retcode = (ulindatalen == uloutdatalen && memcmp(indata,outdata,ulindatalen) == 0) ? 0 : -1;
	if (retcode!=0) goto end;

	/* test2 */
	retcode = ESE_GetSelfCertificate(hSE,devType, devParam,cert,&certlen );
	/*retcode = ESE_GetSelfCertificate(hSE,2, externalEncCertPath,cert,&certlen );*/
	if (retcode!=0) goto end;
	
	ulencdatalen = sizeof(encdata);
	memset(encdata,0,ulencdatalen);
	randomdatalen = sizeof(randomdata);
	memset(randomdata,0,randomdatalen);
	uloutdatalen = sizeof(outdata);
	memset(outdata,0,uloutdatalen);	

	retcode = ESE_SymEncryptEx(hSE,cert,certlen,ulalgid,indata,ulindatalen,encdata,&ulencdatalen,randomdata,&randomdatalen);
	if (retcode!=0) goto end;

	retcode = ESE_SymDecryptEx(hSE,ulalgid,randomdata,randomdatalen,encdata,ulencdatalen,outdata,&uloutdatalen);
	if (retcode!=0) goto end;

	retcode = (ulindatalen == uloutdatalen && memcmp(indata,outdata,ulindatalen) == 0) ? 0 : -1;
	if (retcode!=0) goto end;

end:
	(retcode!=0) ? printf("TestEncDec NG.[0x%x]\n",retcode) : printf("TestEncDec OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestDecodeCert()
{
	HSE hSE = NULL;
	long retcode = -1;	
	int certdevtype;
	char certdevparam[200];
	unsigned char cert[2048]={0};
	unsigned short certlen = sizeof(cert);
	unsigned short dItemNo = 0;

	/* sp */
	unsigned char itemdata[1024]={0};
	unsigned short itemdatalen = sizeof(itemdata);

	printf("\nPlease input certificate device type[HEX]:");
	scanf("%x", &certdevtype);
	printf("\nPlease input certificate device parameter:");
	scanf("%s", certdevparam);

	retcode=ESE_InitialSession(&hSE, certdevtype, "com1", "5b9ab9dd", 0, 2, "", "");
	if (retcode !=0) goto end;

	retcode = ESE_GetSelfCertificate(hSE, certdevtype, certdevparam, cert, &certlen);
	if (retcode !=0) goto end;

	for(dItemNo=1;dItemNo<=75;dItemNo++)
	{
		retcode = ESE_GetCertDetail(hSE,cert,certlen,dItemNo,itemdata,&itemdatalen);
		if (retcode !=0 )
		{
			printf("[%d]:0x%x\n",dItemNo,retcode);
		}
		else
		{
			(itemdatalen > 0) ? printf("[%d]:%s,len=%d\n",dItemNo,itemdata,itemdatalen) : printf("[%d]:NULL\n",dItemNo);			
		}
	}
	retcode = 0;
end:
	(retcode!=0) ? printf("TestDecodeCert NG.[0x%x]\n",retcode) : printf("TestDecodeCert OK.\n");
	return;
}

static void TestVerifCertOnline()
{
	int issuerdevtype=0, certdevtype=0;
	char issuerdevparam[200]={0}, certdevparam[200]={0};
	long retcode = -1;
	HSE hSE = NULL;
	unsigned char cert[2048]={0};
	unsigned short certlen = sizeof(cert);

	printf("\nPlease input certificate device type[HEX]:");
	scanf("%x", &certdevtype);
	printf("\nPlease input certificate device parameter:");
	scanf("%s", certdevparam);
	printf("\nPlease input issuer certificate device type[HEX]:");
	scanf("%x", &issuerdevtype);
	printf("\nPlease input issuer certificate device parameter:");
	scanf("%s", issuerdevparam);

	retcode=ESE_InitialSession(&hSE, 2, "", "", 0, issuerdevtype, issuerdevparam, "");
	if (retcode !=0) goto end;

	retcode = (certdevtype == 2) ? ESE_GetSelfCertificate(hSE, certdevtype, certdevparam, cert, &certlen) : GetCertificate(certdevtype,certdevparam,cert, &certlen);
	if (retcode!=0) goto end;

	retcode=ESE_VerifyCertificateOnline(hSE, cert, certlen);
	if (retcode !=0) goto end;

end:
	(retcode!=0) ? printf("TestVerifCertOnline NG.[0x%x]\n",retcode) : printf("TestVerifCertOnline OK.\n");
	return;
}

static void TestVerifCertOnlineEx()
{
	unsigned long certdevtype = 0;
	long retcode = -1;
	HSE hSE = NULL;
	unsigned char cert[2048]={0};
	unsigned short certlen = sizeof(cert);

	printf("\n豸(eg.Ķ41):");
	scanf("%d", &certdevtype);

	retcode = ESE_InitialSession(&hSE, 2, "", "", 0, 2, "", "");
	if ( retcode != 0 )
	{
		printf("SafeEngineʼʧ,OCSP֤.\n");
		ESE_ClearSession(hSE);
		hSE = NULL;
		return;
	}
	retcode = GetCertificate(certdevtype,"com1",cert, &certlen);
	if ( retcode != 0 )
	{
		printf("豸ȡ֤ʧ,OCSP֤.\n");
		ESE_ClearSession(hSE);
		hSE = NULL;
		return;
	}
	printf("OCSP֤...\n");
	retcode = ESE_VerifyCertificateOnline(hSE, cert, certlen);
	if (retcode == 0)
	{
		printf("֤״̬,OCSP֤.\n");
	}
	else if ( retcode == SE_ECC_ERROR_INVALID_HANDLE || 
		      retcode == SE_ECC_ERROR_INITIAL_SESSION )
	{
		printf("SafeEngineʼ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_MEMORY)
	{
		printf("ڴռ䲻,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_VERIFY_CERT_CHAIN)
	{
		printf("֤֤ʧ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_INVALID_CERT_CHAIN)
	{
		printf("֤ʽ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_INVALID_CERT)
	{
		printf("֤ʽ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_CERT_EXPIRED)
	{
		printf("֤ѹ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_GET_CERT_DETAIL)
	{
		printf("֤ȡOCSPַʧ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_CONNECT_OCSP)
	{
		printf("OCSPʧ,OCSP֤.\n");
	}
	else if (retcode == SE_ECC_ERROR_CERT_REVOKED)
	{
		printf("֤ѳ,OCSP֤.\n");
	}
	else
	{
		printf("δ֪,OCSP֤.\n");
	}
	ESE_ClearSession(hSE);
	hSE = NULL;
}

static void TestBASE64EncDec()
{
	long retcode = 0;
	unsigned char indata[4096]={0};
	unsigned long ulindatalen = sizeof(indata);
	unsigned char base64data[4096]={0};
	unsigned long ulbase64datalen = sizeof(base64data);
	unsigned char strinfile[MAX_PATH] = {0};
	int dEncDecFlag = 0;

	printf("\n enc or dec(enc=1;dec=2) :");
	scanf("%d", &dEncDecFlag);
	printf("\ninput input file data:");
	scanf("%s", strinfile);
	if (dEncDecFlag != 1 && dEncDecFlag != 2)
	{
		retcode = -1;
		goto end;
	}
	retcode = ReadDiskFile(strinfile,indata,&ulindatalen);
	if (retcode !=0) goto end;		

	/* base64 enc */
	if (dEncDecFlag == 1)
	{
		retcode = ESE_PEMEncode(NULL,indata,ulindatalen,base64data,&ulbase64datalen);
		if (retcode !=0) goto end;
		/* write base64 data to file */
		retcode = File_writeEx("encBASE64.txt",base64data,ulbase64datalen);
		if (retcode !=0) goto end;
	}
	/* base64 dec */
	if (dEncDecFlag == 2)
	{
		retcode = ESE_PEMDecode(NULL,indata,ulindatalen,base64data,&ulbase64datalen);
		if (retcode !=0) goto end;
		/* write dec data to file */
		retcode = File_writeEx("decBASE64.dat",base64data,ulbase64datalen);
		if (retcode !=0) goto end;
	}

end:
	(retcode!=0) ? printf("TestBASE64EncDec NG.[0x%x]\n",retcode) : printf("TestBASE64EncDec OK.\n");
	return;
}

static void TestVerifyCert()
{
	HSE hSE = NULL;
	int devType = 0;
	char param[260]={0};
	int certDevType = 0;
	char certDevParam[260]={0};
	int certchainDevType = 0;
	char certchainDevParam[260]={0};

	int mode=0;
	long retcode = -1;
	unsigned char cert[2048];
	unsigned short certLen = sizeof(cert);

	printf("\ninput cert device type[HEX]:");
	scanf("%x", &certDevType);
	printf("\ninput cert device parameter:");
	scanf("%s", certDevParam);
	printf("\ninput certchain device type[HEX]:");
	scanf("%x", &certchainDevType);
	printf("\ninput certchain device parameter:");
	scanf("%s", certchainDevParam);
	printf("\ninput verify mode:");
	scanf("%d", &mode);

	CHKINPUT(certDevParam);
	CHKINPUT(certchainDevParam);

	retcode = ESE_InitialSession(&hSE, 2, "", "", 0, certchainDevType, certchainDevParam, "");
	if (retcode!=0) goto end;

	retcode = (certDevType == 2) ? ESE_GetSelfCertificate(hSE, certDevType, certDevParam, cert, &certLen) : GetCertificate(certDevType,certDevParam,cert, &certLen);
	if (retcode!=0) goto end;

	ESE_SetConfiguration(hSE, mode);
	retcode = ESE_VerifyCertificate(hSE, cert, certLen); 
	if (retcode!=0) goto end;

end:
	(retcode!=0) ? printf("TestVerifyCert NG.[0x%x]\n",retcode) : printf("TestVerifyCert OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestEnvelope()
{
	HSE hSE=NULL,hSE_file=NULL,hSE_sdf = NULL;
	unsigned char out[4096];
	unsigned long outlength = sizeof(out);
	unsigned char cert[2048];
	unsigned short certlength = sizeof(cert);
	long retcode = -1;
	unsigned char indata[] = "123456789012345";
	unsigned long inlength = sizeof(indata);
	unsigned char outdata[100];
	unsigned long outdatalen = sizeof(outdata);

	long devType = 0;
	char devParam[260];
	long certDevType = 0;
	char certDevParam[260];
	char password[256];

	int algtype = 0;

	printf("\ninput device type[HEX]:");
	scanf("%x", &devType);
	printf("\ninput device parameter:");
	scanf("%s", devParam);
	printf("\ninput password:");
	scanf("%s", password);
	printf("\ninput cert device type[HEX]:");
	scanf("%x", &certDevType);
	printf("\ninput cert device parameter:");
	scanf("%s", certDevParam);
	printf("\ninput alg type(0:SM1,1:AES,2:SM4):");
	scanf("%d", &algtype);

	retcode = ESE_InitialSession(&hSE, devType, devParam, password, 0, 2, "", "");
	if (retcode!=0){
		printf("ESE_InitialSession error.[0x%x]\n",retcode);
		goto end;
	}
	certlength = sizeof(cert);
	memset(cert,0,certlength);
	retcode = ESE_GetSelfCertificate(hSE, certDevType, certDevParam, cert, &certlength);
	if (retcode!=0){
		printf("ESE_GetSelfCertificate error.[0x%x]\n",retcode);
		goto end;
	}

	outlength = sizeof(out);
	memset(out,0,outlength);
	if (algtype == 0){
		retcode = ESE_Envelope_P7(hSE, indata, inlength, SE_ALG_sm1, SE_ALG_ecSM2_3Encryption, out, &outlength, cert, certlength);
	}
	else if (algtype == 1){
		retcode = ESE_Envelope_P7(hSE, indata, inlength, SE_ALG_aes, SE_ALG_ecSM2_3Encryption, out, &outlength, cert, certlength);
	}
	else{
		retcode = ESE_Envelope_P7(hSE, indata, inlength, "1.2.156.10197.1.104", SE_ALG_ecSM2_3Encryption, out, &outlength, cert, certlength);
	}

	if (retcode!=0)
	{
		printf("ESE_Envelope_P7 error.[0x%x]\n",retcode);
		goto end;
	}

	File_writeEx("td_envelope.asn1",out,outlength);

	outdatalen = sizeof(outdata);
	memset(outdata,0,outdatalen);
	retcode = ESE_DecodeEnvelope_P7(hSE, out, outlength, outdata, &outdatalen);
	if (retcode!=0)
	{
		printf("ESE_DecodeEnvelope_P7 error.[0x%x]\n",retcode);
		goto end;
	}

	if (outdatalen != inlength || memcmp(outdata,indata,inlength) != 0) retcode = -1;
	if (retcode!=0) goto end;
			
end:
	(retcode!=0) ? printf("TestEnvelope NG.[0x%x]\n",retcode) : printf("TestEnvelope OK.\n");
	ESE_ClearSession(hSE);
	hSE = NULL;
	return;
}

static void TestEncodeCert()
{
	HSE hSE = NULL;	
	long retcode = -1;
	int priKeyType=0;
	unsigned char pubkey[1024]={0}, prikey[1024]={0};
	int pubkeylen=1024, prikeylen=1024;
	unsigned short usPubkeylen,usPrikeylen=1024;
	char priKeyParam[260]={0};
	char g_prikeydecpass[K_PRIKEY_LEN]={0};
	int dKeyPairType = 0; /* [0:sign];[1:enc] */
	/* sd */
	X500_NAME_INFO s_CertInfoEx={0};
	unsigned short s_certinfodatalen = sizeof(g_certinfodata);
	unsigned long ulcertinfodatalen =  sizeof(g_certinfodata);
	char filepath[256]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key user password:");
	scanf("%s", g_prikeydecpass);
	printf("\ninput cert type[0:sign,1:enc]:");
	scanf("%d", &dKeyPairType);

	retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
	if (retcode!=0) goto end;

	strncpy(s_CertInfoEx.commonName,"veniwang",8);
	strncpy(s_CertInfoEx.countryName,"й",4);
	strncpy(s_CertInfoEx.organizationName,"sheca",5);
	strncpy(s_CertInfoEx.organizationalUnitname,"ϺCA",6);
	strncpy(s_CertInfoEx.stateOrProvinceName,"Ϻ",4);	
	strncpy(s_CertInfoEx.localityName,"shanghai",8);
	strncpy(s_CertInfoEx.title,"ABC",3);
	strncpy(s_CertInfoEx.surname,"wang",4);
	strncpy(s_CertInfoEx.givenName,"yulin",5);
	strncpy(s_CertInfoEx.initials,"XYZ",3);
	strncpy(s_CertInfoEx.email,"abc@126.com",11);
	strncpy(s_CertInfoEx.postalAddress,"hongkou",7);
	strncpy(s_CertInfoEx.postalCode,"200080",6);
	strncpy(s_CertInfoEx.postalOfficeBox,"Ĵ·",8);
	strncpy(s_CertInfoEx.telephoneNumber,"36390000",8);
	strncpy(s_CertInfoEx.telexNumber,"36390001",8);
	
	retcode = ESE_EncodeCert(hSE,s_CertInfoEx,dKeyPairType,g_certinfodata,&s_certinfodatalen);
	if (retcode!=0) goto end;
	ulcertinfodatalen = s_certinfodatalen;

	printf("\ninput save path:");
	scanf("%s", filepath);
	retcode = File_writeEx(filepath,g_certinfodata,ulcertinfodatalen);

end:
	(retcode!=0) ? printf("TestEncodeCert NG.[0x%x]\n",retcode) : printf("TestEncodeCert OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestSKFdeviceGenKeyPair()
{
	HSE hSE = NULL;	
	long retcode = -1;
	int priKeyType=0;
	unsigned char pubkey[1024]={0}, prikey[1024]={0};
	int pubkeylen=1024, prikeylen=1024;
	unsigned short usPubkeylen,usPrikeylen=1024;
	char priKeyParam[260]={0};
	char g_prikeydecpass[20]={0};
	/* spec define */
	int dKeyPairType = 0; /* [0:sign];[1:enc] */
	HSE hSE_ssl = NULL;

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput private key user password:");
	scanf("%s", g_prikeydecpass);

	dKeyPairType = (priKeyType & 0x00FF0000) >> 16;

	if (dKeyPairType == 0){
		/* gen sign key pair */
		retcode = ESE_InitialSession(&hSE,priKeyType,priKeyParam,g_prikeydecpass,0,2,"","");
		if (retcode!=0) goto end;
		retcode = ESE_GenKeyPair(hSE,"",g_prikeydecpass,SE_ALG_ecsm2WithSM3,SE_EC_sm2,pubkey,&pubkeylen,prikey,&prikeylen);
		if (retcode!=0) goto end;
	}
	else if (dKeyPairType == 1){
		/* gen enc key pair */
		retcode = ESE_InitialSession(&hSE_ssl,2,"","",0,2,"","");
		if (retcode!=0)
		{
			ESE_ClearSession(hSE_ssl);
			goto end;
		}
		retcode = ESE_GenKeyPair(hSE_ssl,"","",SE_ALG_ecsm2WithSM3,SE_EC_sm2,pubkey,&pubkeylen,prikey,&prikeylen);
		if (retcode!=0)
		{
			ESE_ClearSession(hSE_ssl);
			goto end;
		}
		usPubkeylen = pubkeylen;
		usPrikeylen = prikeylen;
		retcode = WriteKeyPair(SE_EC_sm2,priKeyType,priKeyParam,g_prikeydecpass,prikey,usPrikeylen,pubkey,usPubkeylen);
		if (retcode!=0)
		{
			ESE_ClearSession(hSE_ssl);
			goto end;
		}
	}else{
		/* do noyhing */
	}

end:
	(retcode!=0) ? printf("TestSKFdeviceGenKeyPair NG.[0x%x]\n",retcode) : printf("TestSKFdeviceGenKeyPair OK.\n");
	ESE_ClearSession(hSE);
	return;
}

static void TestSKFdeviceInit()
{
	HSE hSE = NULL;
	long retcode = -1;
	unsigned char pubkey[1024], prikey[1024];
	int pubkeylen, prikeylen;
	char priKeyParam[260];
	int priKeyType;
	char password[20]={0};
	char adminpass[20];
	char userpass[20];
	int dType = 0;
	/* <--make p10 info */
	FILE *fp;
	char file[256];
	PKCS10_INFO p10info;
	unsigned char p10[8192];
	int p10len;
	int encodetype;
	unsigned char p10pem[8192];
	int p10pemLen = 8192;
	unsigned long ldevicetype = 0;
	char pripass[100];
	char sn[64] = {0};
	char snchk[64] = {0};

char pbAuthKeyOld[16]={0x32, 0x45, 0x60, 0x87, 0xac, 0x3d, 0x76, 0xcd, 0x79, 0xa2, 0x66, 0x55, 0x40, 0x83, 0x79, 0x3f};


	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter: ");
	scanf("%s", priKeyParam);
	printf("\ninput private key device password:");
	scanf("%s", password);
	printf("\ninput private key admin password:");
	scanf("%s", adminpass);
	printf("\ninput private key user password:");
	scanf("%s", userpass);

//	memcpy(password,pbAuthKeyOld,16);
#if 0
	printf("\ninput device sn:");
	scanf("%s", snchk);

	retcode = GetSerialNumber(priKeyType,priKeyParam,sn);
	if ( retcode != 0 || memcmp(snchk,sn,strlen(sn)) != 0 )
	{
		printf("serial number error.\n");
		return;
	}
#endif
	retcode = InitICCard(priKeyType,priKeyParam,password,adminpass,14,userpass,14);
	if ( retcode !=0 )
	{
		printf("InitICCard error.0x%x\n",retcode);
		return;
	}
	printf("TestSKFdeviceInit OK.\n");

	return;
}

static void TestSign()
{
	char priKeyParam[260]={0};
	int priKeyType;
	char password[100]={0};
	char srcData[] = "abcd";
	char dstData[2048]={0};
	char digest[2048]={0};
	unsigned long digestLen=2048;
	unsigned long signedLen=2048;
	HSE hSE = NULL;
	HSE hSE_vf = NULL;
	char certDevParam[260]={0};
	int certDevType;
	unsigned char cert[2048]={0};
	unsigned short certLen = sizeof(cert);
	long retcode = -1;
	unsigned long orignalDataLen = sizeof(srcData);
	unsigned char *bigData = NULL;
	unsigned long bigDataLen = 0;
	unsigned char filename[MAX_PATH]={0};

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput password:");
	scanf("%s", password);
	printf("\ninput certificate device type(HEX):");
	scanf("%x", &certDevType);
	printf("\ninput certificate device parameter:");
	scanf("%s", certDevParam);
	printf("\ninput file path:");
	scanf("%s", filename);

	CHKINPUT(password);

	retcode = ESE_InitialSession(&hSE, priKeyType, priKeyParam, password, 0, 2, "", "");
	if(retcode != 0){
		printf("initialsession fail, retcode:%08X\n", retcode);
		goto end;
	}
	retcode = ESE_GetSelfCertificate(NULL, certDevType, certDevParam, cert, &certLen);
	if(retcode != 0){
		printf("get self certificate fail, retcode: %08X\n", retcode);
		goto end;
	}
#if 1
	bigDataLen = 3;
	bigData = (unsigned char*)malloc(bigDataLen);
	retcode = ESE_GenRandomBytes(hSE,bigData,bigDataLen);

	retcode = ESE_SignData(hSE, (unsigned char*)srcData, sizeof(srcData), SE_ALG_ecsm2WithSM3, (unsigned char*)dstData, &signedLen);
	/*retcode = ESE_SignData(hSE, bigData, bigDataLen, SE_ALG_ecsm2WithSM3, (unsigned char*)dstData, &signedLen);*/
	if(retcode != 0){
		printf("sign data fail, retcode:%08X\n", retcode);
		goto end;
	}
	else{
		retcode = File_writeEx("c:\\signdata1.dat",dstData,(unsigned long)signedLen);
		printf("success to sign data.\n");
	}
	retcode = ESE_VerifySignData(hSE, (unsigned char*)srcData, sizeof(srcData), SE_ALG_ecsm2WithSM3, (unsigned char*)dstData, signedLen, cert, (unsigned short)certLen);
	if(retcode != 0){
		printf("verify signature fail:%08X\n", retcode);
		goto end;
	}
	else{
		printf("success to verify signature.\n");
	}

	signedLen = 1024;
	retcode = ESE_SignData_P7(hSE, (unsigned char*)srcData, sizeof(srcData), SE_ALG_ecsm2WithSM3, cert, certLen,(unsigned char*)dstData, &signedLen);
	if(retcode != 0){
		printf("sign data fail, retcode:%08X\n", retcode);
		goto end;
	}
	else{
		printf("success to sign data.\n");
	}
	digestLen = 1024;
	retcode = ESE_VerifySignData_P7( hSE, 
									 dstData, 
									 signedLen, 
									 (unsigned char*)srcData, 
									 &orignalDataLen, 
									 cert, 
									 &certLen,
									 digest,
									 &digestLen);
	if(retcode != 0){
		printf("verify signature fail:%08X\n", retcode);
		goto end;
	}
	else{
		printf("success to verify signature.\n");
	}
#endif	
	printf("Being large file signature...\n");
	signedLen = 2048;
	retcode = ESE_SignFile(hSE,filename,SE_ALG_sha1,dstData,&signedLen);
	if (retcode!=0) goto end;
	printf("ESE_SignFile OK.\n");

	printf("Being verified large file signature...\n");
	retcode = ESE_InitialSession(&hSE_vf,2,"","",0,2,"","");
	if (retcode!=0) goto end;
	retcode = ESE_VerifySignFile(hSE_vf,filename,SE_ALG_sha1,dstData,signedLen,cert,certLen);
	if (retcode!=0) goto end;
	ESE_ClearSession(hSE_vf);
	printf("ESE_VerifySignFile OK.\n");

end:
	if (bigData) free(bigData);
	ESE_ClearSession(hSE);
	if (retcode!=0) printf("Test Sign error.[0x%8x]\n",retcode);
	return;
}

static void TestLargeDataEncDec()
{
	long retcode = -1;
	char srcFile[MAX_PATH]={0};
	char encFile[MAX_PATH]={0};
	char decFile[MAX_PATH]={0};
	unsigned char signdata[1024]={0};
	unsigned long signdatalen = 1024;
	unsigned char certinfo[4096]={0};
	unsigned short certinfolen = 4096;
	HSE hSE = NULL;
	unsigned char symkey[16]={0};
	int symkeylen = 16;
#if 0
	int i = 1;
	for (i=1;i<=12;i++)
	{
		memset(srcFile,0,MAX_PATH);
		memset(encFile,0,MAX_PATH);
		memset(decFile,0,MAX_PATH);
		 _snprintf(srcFile, sizeof(srcFile), "E:\\TestEncFile\\%d.txt", i);
		 _snprintf(encFile, sizeof(encFile), "E:\\TestEncFile\\test%d.dat", i);
		 _snprintf(decFile, sizeof(decFile), "E:\\TestEncFile\\tres%d.txt", i);
		retcode = ESE_SymEncFile(NULL,"1234567812345678",16,srcFile,NULL,encFile);
		if (retcode != 0) goto end;
		retcode = ESE_SymDecFile(NULL,"1234567812345678",16,encFile,NULL,decFile);
		if (retcode != 0) goto end;
	}
#endif
	printf("\ninput file path:");
	scanf("%s",srcFile);
	_snprintf(encFile, sizeof(encFile), "%s.enc", srcFile);
	_snprintf(decFile, sizeof(decFile), "%s.dec", srcFile);
	/* test start. */
	retcode = ESE_InitialSession(&hSE,41,"com1","sheca123",0,2,"","");
	if (retcode != 0) goto end;	
	retcode = ESE_GetSelfCertificate(hSE,41,"com1",certinfo,&certinfolen);
	if (retcode != 0) goto end;
	printf("Being Sign File...\n");
	retcode = ESE_SignFile(hSE,srcFile,SE_ALG_sha1,signdata,&signdatalen);
	if (retcode != 0) goto end;
	printf("Being Verify Signed File...\n");
	retcode = ESE_VerifySignFile(hSE,srcFile,SE_ALG_sha1,signdata,signdatalen,certinfo,certinfolen);
	if (retcode != 0) goto end;
	retcode = ESE_GenRandomBytes(hSE,symkey,symkeylen);
	if (retcode != 0) goto end;
	printf("Being SymEnc File...\n");
	retcode = ESE_SymEncFile(NULL,symkey,symkeylen,srcFile,NULL,encFile);
	if (retcode != 0) goto end;
	printf("Being SymDec File...\n");
	retcode = ESE_SymDecFile(NULL,symkey,symkeylen,encFile,NULL,decFile);
	if (retcode != 0) goto end;
	/* test end. */
end:
	ESE_ClearSession(hSE);
	(retcode!=0) ? printf("TestLargeDataEncDec error.[0x%8x]\n",retcode) : printf("TestLargeDataEncDec OK.\n");
#if 0
	char priKeyParam[260]={0};
	int priKeyType;
	char password[100]={0};
	char srcData[] = "abcd";
	char dstData[2048]={0};
	char digest[2048]={0};
	unsigned long digestLen=2048;
	unsigned long signedLen=2048;
	HSE hSE = NULL;
	HSE hSE_enc = NULL;
	char certDevParam[260]={0};
	int certDevType;
	unsigned char cert[2048]={0};
	unsigned short certLen = sizeof(cert);
	long retcode = -1;
	unsigned long orignalDataLen = sizeof(srcData);

	unsigned char *bigData = NULL;
	unsigned long bigDataLen = 0;
	int bigDatalength = 0;
	unsigned char filename[MAX_PATH]={0};

	/* Enc Dec Data */
	unsigned char *outdata = NULL;
	int outdatalen = 0;
	unsigned char cipherdata[64]={0};
	int cipherdatalen = 64;
	unsigned char ciphersession[256]={0};
	int ciphersessionlen = 256;

	printf("\ninput private key type(HEX):");
	scanf("%x", &priKeyType);
	printf("\ninput private key parameter:");
	scanf("%s", priKeyParam);
	printf("\ninput password:");
	scanf("%s", password);
	printf("\ninput certificate device type(HEX):");
	scanf("%x", &certDevType);
	printf("\ninput certificate device parameter:");
	scanf("%s", certDevParam);
	printf("\ninput file path:");
	scanf("%s", filename);

	retcode = ESE_InitialSession(&hSE, priKeyType, priKeyParam, password, 0, 2, "", "");
	if(retcode != 0){
		printf("initialsession fail, retcode:%08X\n", retcode);
		goto end;
	}
	retcode = ESE_GetSelfCertificate(hSE, certDevType, certDevParam, cert, &certLen);
	if(retcode != 0){
		printf("get self certificate fail, retcode: %08X\n", retcode);
		goto end;
	}
	retcode = ReadDiskFile(filename,NULL,&bigDataLen);
	if(retcode != 0){
		printf("ReadDiskFile fail, retcode:%08X\n", retcode);
		goto end;
	}
	bigData = (unsigned char*)malloc(bigDataLen);
	memset(bigData,0,bigDataLen);
	retcode = ReadDiskFile(filename,bigData,&bigDataLen);
	if(retcode != 0){
		printf("ReadDiskFile fail, retcode:%08X\n", retcode);
		goto end;
	}
	outdatalen = bigDataLen + 1024;
	outdata = (unsigned char*)malloc(outdatalen);
	memset(outdata,0,outdatalen);

	/* enc */
printf("begin to Enc large data...\n");
	retcode = ESE_InitialSession(&hSE_enc,2,"","",0,2,"","");
	retcode = ESE_LargeDataEnc(hSE_enc,cert,certLen,0x401,bigData,(int)bigDataLen,outdata,&outdatalen,cipherdata,&cipherdatalen,ciphersession,&ciphersessionlen);
	if(retcode != 0){
		printf("ESE_LargeDataEnc fail, retcode:%08X\n", retcode);
		goto end;
	}
	ESE_ClearSession(hSE_enc);
printf("saving large enc data...\n");
	retcode = File_writeEx("enc.dat",outdata,(unsigned long)outdatalen);
		
	/* dec */
printf("begin to Dec large data...\n");
	bigDatalength = bigDataLen;
	memset(bigData,0,bigDatalength);
	retcode = ESE_LargeDataDec(hSE,0x401,cipherdata,cipherdatalen,ciphersession,ciphersessionlen,outdata,outdatalen,bigData,&bigDatalength);
	if(retcode != 0){
		printf("ESE_LargeDataDec fail, retcode:%08X\n", retcode);
		goto end;
	}
printf("saving large dec data...\n");
	retcode = File_writeEx("dec.dat",bigData,(unsigned long)bigDatalength);

end:
	if (bigData) free(bigData);
	if (outdata) free(outdata);
	ESE_ClearSession(hSE);
	(retcode!=0) ? printf("TestLargeDataEncDec error.[0x%8x]\n",retcode) : printf("TestLargeDataEncDec OK.\n");
#endif
	return;
}

static void TestSignFile()
{

}
static void TestVerifySignFile()
{
}

static void TestEncEnvelopeFile()
{
#if 0
	long retcode = 0;
	HSE hSE = NULL;
	char filepath[MAX_PATH]={0};
	unsigned long certDevType = 0;
	char certDevParam[MAX_PATH]={0};
	char certDevPwd[100] = {0};

	unsigned char cert[2048];
	unsigned short certlength = sizeof(cert);

	unsigned char cipherdata[64] = {0};
	int cipherdatalen = sizeof(cipherdata);
	unsigned char ciphersession[1024] = {0};
	int ciphersessionlen = sizeof(ciphersession);

	/* input */
	unsigned char *bigdata = NULL;
	unsigned long ulbigdatalen = 0;
	/* output */
	unsigned char *outdata = NULL;
	int outdatalen = 0;
	/* input test param */
	printf("\ninput file path:");
	scanf("%s", filepath);
	printf("\ninput cert device type[HEX]:");
	scanf("%x", &certDevType);
	printf("\ninput cert parameter:");
	scanf("%s", certDevParam);
	/* enc start */
	if (certDevType != 2)
	{
		printf("\ninput cert device password:");
		scanf("%s", certDevPwd);
		retcode = ESE_InitialSession(&hSE,certDevType,certDevParam,certDevPwd,0,2,"","");
	}
	else
	{
		retcode = ESE_InitialSession(&hSE,2,"","",0,2,"","");
	}	
	if (retcode != 0) goto end;
	/* get sm2 certificate */
	retcode = ESE_GetSelfCertificate(hSE, certDevType, certDevParam, cert, &certlength);
	if (retcode != 0) goto end;	
	/* get file length */
	retcode = ReadDiskFile(filepath,NULL,&ulbigdatalen);
	if (retcode != 0) goto end;	
	/* allocate memory space */
	bigdata = (unsigned char*)malloc(ulbigdatalen);
	memset(bigdata,0,ulbigdatalen);
	/* get the contents of the file */
	retcode = ReadDiskFile(filepath,bigdata,&ulbigdatalen);
	if (retcode != 0) goto end;

	/* set the output buffer size.(1Kbytes are reserved) */
	outdatalen = ulbigdatalen + 1024;
	outdata = (unsigned char*)malloc(outdatalen);
	memset(outdata,0,outdatalen);
	
printf("begine to enc big file...\n");
	retcode = ESE_LargeDataEnc(hSE,cert,certlength,SGD_SDBI_CRYPTY,bigdata,(int)ulbigdatalen,outdata,&outdatalen,cipherdata,&cipherdatalen,ciphersession,&ciphersessionlen);
	if (retcode != 0) goto end;
printf("saving enc data...\n");
	retcode = File_writeEx("encFile.dat",outdata,(unsigned long)outdatalen);
	retcode = File_writeEx("cipherData.dat",cipherdata,(unsigned long)cipherdatalen);
	retcode = File_writeEx("cipherSession.dat",ciphersession,(unsigned long)ciphersessionlen);

end:
	if (bigdata) free(bigdata);
	if (outdata) free(outdata);
	(retcode!=0) ? printf("TestEncBigFile NG.[0x%x]\n",retcode) : printf("TestEncBigFile OK.\n");
	ESE_ClearSession(hSE);
#endif
	return;
}

static void TestDecEnvelopeFile()
{
#if 0
	long retcode = 0;
	HSE hSE = NULL;
	unsigned long prikey = 0;
	unsigned char prikeyparm[260]={0};
	unsigned char prikeypass[260]={0};
	/* input */
	unsigned char *bigdata = NULL;
	unsigned long ulbigdatalen = 0;
	/* output */
	unsigned char *outdata = NULL;
	int outdatalen = 0;
	/* enc param */
	unsigned char cipherdata[64]={0};
	unsigned long ulcipherdatalen = sizeof(cipherdata);
	
	unsigned char ciphersession[1024]={0};
	unsigned long ulciphersessionlen = sizeof(ciphersession);

	printf("\ninput private key type(HEX):");
	scanf("%x", &prikey);
	printf("\ninput private key parameter:");
	scanf("%s", prikeyparm);
	printf("\ninput password:");
	scanf("%s", prikeypass);

	/* dec start */
	retcode = ESE_InitialSession(&hSE,prikey,prikeyparm,prikeypass,0,2,"","");
	if (retcode != 0) goto end;

	/* get enc file length */
	retcode = File_readEx("encFile.dat",NULL,&ulbigdatalen);
	/* allocate memory space */
	bigdata = (unsigned char*)malloc(ulbigdatalen);
	memset(bigdata,0,ulbigdatalen);
	/* get the contents of the enc file */
	retcode = File_readEx("encFile.dat",bigdata,&ulbigdatalen);
	if (retcode != 0) goto end;
	
	/* get cipher data */
	retcode = File_readEx("cipherData.dat",cipherdata,&ulcipherdatalen);
	if (retcode != 0) goto end;

	/* get cipher session */
	retcode = File_readEx("cipherSession.dat",ciphersession,&ulciphersessionlen);
	if (retcode != 0) goto end;

	/* dec start */
printf("begine to dec big file...\n");
	/* malloc dec data memory.(1Kbytes are reserved) */
	outdatalen = ulbigdatalen + 1024;
	outdata = (unsigned char*)malloc(outdatalen);
	memset(outdata,0,outdatalen);
	retcode = ESE_LargeDataDec(hSE,SGD_SDBI_CRYPTY,cipherdata,(int)ulcipherdatalen,ciphersession,(int)ulciphersessionlen,bigdata,(int)ulbigdatalen,outdata,&outdatalen );
	if (retcode != 0) goto end;
printf("saving dec data...\n");
	retcode = File_writeEx("decFile.dat",outdata,(unsigned long)outdatalen);

end:
	if (bigdata) free(bigdata);
	if (outdata) free(outdata);
	(retcode!=0) ? printf("TestDecEnvelopeFile NG.[0x%x]\n",retcode) : printf("TestDecEnvelopeFile OK.\n");
	ESE_ClearSession(hSE);
#endif
	return;
}

#ifdef _LINUX
int main()
#else
void main()
#endif
{
    HSE hSE = NULL;
	int dChocis = 0;
    int pattern_num = 0;
    int errcode = 0;
    long resultAct = 0;
    long lModuleNo, lVersion;
    LOAD_SAFEENGINE_DLL();
    LOAD_SAFEENGINE_API(g_sl_safeengine);
    LOAD_READCERT_API(g_sl_readcert);
    ESE_ShowVersion(NULL, &lModuleNo, &lVersion);
//	printf("The current test version is 0x%x,0x%x\n\n",lModuleNo,lVersion);
	while(1)
	{
#ifdef _WINDOWS
		_flushall();
#else
		fflush(stdin);
		fflush(stdout);
		fflush(stderr);
#endif

#ifndef OCSP_TEST
		printf("==============================================================================\n");
		printf("=                                                                            =\n");
		printf("=                        UniTrust SafeEngine Tools Program                   =\n");
		printf("=                   Copyright 2010 SHECA. All rights reserved.               =\n");
		printf("=                                                                            =\n");
		printf("==============================================================================\n");
		printf("\n");
		printf("Please choose functions.\n");
		printf("1. Sign and VerifySign.\t\t\t");
		printf("2. Envelope and decode Envelope.\n");
		printf("3. Verify Certificate.\t\t\t");
		printf("4. Verify Certificate Online (OCSP).\n");
		printf("5. Decode Certificate.\t\t\t");
		printf("6. Enc and Dec.\n");
		printf("7. Get Cert Info by OID\t\t\t");
		printf("8. Encode PKCS#12.\n");
		printf("9. Get Certificate Online.\t\t");
		printf("10.Change Password.\n");
		printf("11.BASE64 Encode and Decode.\t\t");
		printf("12.Multi thread random test.\n");
		printf("13.Show device info.\t\t\t");
		printf("14.Show all supported device info.\n");
		printf("15.Write Certificate & key to device.\t");
		printf("16.Gen key pair & encode pkcs10.\n");
		printf("17.Enocde Cert.\t\t\t\t");
		printf("18.Show Version.\n");
		printf("19.Asymmetric enc & dec.\t\t");
		printf("20.Large data test.\n");
		printf("21.Performance test.\t\t\t");
		printf("22.Lock test.\n");
		printf("23.Key pair test.\t\t\t");
		printf("24.Device test.\n");
		printf("25.Decode PKCS10.\t\t\t");
		printf("26.Seal test.\n");
		printf("27.Envelope and decode by pubkey.\t");
		printf("28.TestDevSerialNumber.\n");	
		printf("29.TestSKFdeviceInit.\t\t\t");
		printf("30.TestSKFdeviceGenKeyPair.\n");	
		printf("31.TestSKFdeviceWriteFile.\t\t");
		printf("32.TestSKFdeviceReadFile.\n");
		printf("33.TestSignFile.\t\t\t");
		printf("34.TestVerifySignFile.\n");
		printf("35.TestEncEnvelopeFile.\t\t\t");
		printf("36.TestDecEnvelopeFile.\n");
		printf("0. Exit.\n");
		printf("\n");
		printf("Please enter your choice:");
		scanf("%d", &dChocis);

		if (dChocis == 0){
#ifdef _LINUX
			return 0;
#else
			return;
#endif
		}
		else if(dChocis == 1){
			TestSign();
		}else if(dChocis == 2){
			TestEnvelope();
		}else if(dChocis == 3){
			TestVerifyCert();
		}else if(dChocis == 4){
			TestVerifCertOnline();
		}else if(dChocis == 5){
			TestDecodeCert();
		}else if(dChocis == 6){
			TestEncDec();
		}else if(dChocis == 7){
			TestGetCertInfoByOID();
		}else if(dChocis == 8){
			TestEncDecPkcs12();
		}else if(dChocis == 9){
			TestGetCertOnline();
		}else if(dChocis == 10){
			TestChangePassword();
		}else if(dChocis == 11){
			TestBASE64EncDec();
		}else if(dChocis == 12){
			TestMultiThread();
		}else if(dChocis == 13){
			TestDeviceInfo();
		}else if(dChocis == 14){
			TestSupportedDeviceInfo();
		}else if(dChocis == 15){
			TestWriteData2Device();
		}else if(dChocis == 16){
			TestEncDecPkcs10();
		}else if(dChocis == 17){
			TestEncodeCert();
		}else if(dChocis == 18){
			TestShowVersion();					
		}else if(dChocis == 19){
			TestMutiEncDecP10();					
		}else if(dChocis == 20){
			TestLargeDataEncDec();					
		}else if(dChocis == 25){
			TestDecodeP10();
		}else if(dChocis == 27){
			TestGetPublicKey();
		}else if(dChocis == 29){
			TestSKFdeviceInit();
		}else if(dChocis == 30){
			TestSKFdeviceGenKeyPair();
		}else if (dChocis == 32){
			TestGetDataFromDevice();
		}else if(dChocis == 22){
			TestUnLockDevice();
		}else if(dChocis == 19){
			TestVerifCertOnlineEx();
		}else if(dChocis == 33){
			TestSignFile();
		}else if(dChocis == 34){
			TestVerifySignFile();
		}else if(dChocis == 35){
			TestEncEnvelopeFile();
		}else if(dChocis == 36){
			TestDecEnvelopeFile();
		}
		else{
			printf("Wrong choice.\n");
		}
#else
	TestVerifCertOnlineEx();
#endif
	}

#ifdef _LINUX
	return 0;
#else
	return;
#endif
}
